<?php

class ReedsReader {

  /**
   * Title of this reader
   */
  public function title() {
    return "XML Importer";
  }

  /**
   * Config form callback
   */
  static public function configForm(&$form, &$form_state, &$data) {
    $form['xpath_base'] = array(
      '#title' => t('XPath base expression'),
      '#description' => t('This can be for example /nodes/node when structure is &lt;nodes&gt;&lt;node&gt;'),
      '#type' => 'textfield',
      '#default_value' => $data['xpath_base'],
    );
  }

  /**
   * Config form submit callback
   */
  static public function configFormSubmit(&$form, &$form_state, &$data) {
    $data['xpath_base'] = $form_state['input']['xpath_base'];
  }

  /**
   * Config form item callback
   */
  static public function configFormItem($config, $key, $row) {
    return array(
      $key . '-xpath' => array(
        '#type' => 'textfield',
        '#title' => 'XPath expression',
        '#default_value' => isset($row['xpath']) ? $row['xpath'] : '',
      )
    );
  }

  /**
   * config item form submit callback
   */
  static public function configFormItemSubmit($config, $input, $key, &$row) {
    $row['xpath'] = $input[$key . '-xpath'];
  }

  /**
   * Status form callback
   */
  static public function statusForm(&$form, &$form_state, &$config) {
    $form['file_path'] = array(
      '#title' => t('File'),
      '#description' => t('URL or file path.'),
      '#type' => 'textfield',
      '#default_value' => isset($config->data['file_path']) ? $config->data['file_path'] : '',
    );
  }

  /**
   * Status form submit
   */
  static public function statusFormSubmit(&$form, &$form_state, &$config) {
    $config->data['file_path'] = $form_state['input']['file_path'];
  }

  /**
   * Run reed configuration
   */
  static public function run(&$config) {
    return array(
      'failed' => 0,
      'new' => 0,
      'updated' => 0,
    );
  }

  /**
   *
   */
  function createUpdateEntity($entity_type, $bundle, $lang, $id_fields, $data) {

    $info = entity_get_info($entity_type);
    $bundle_key = $info['entity keys']['bundle'];

    // fetch entity by ids or create a new one
    $entity_field_query = new EntityFieldQuery();
    $entity_field_query->entityCondition('entity_type', $entity_type);

    foreach ($id_fields as $field => $value) {
      $entity_field_query->propertyCondition($field, $value);
    }

    $entities = $entity_field_query->execute();

    if (!empty($entities) && isset($entities[$entity_type])) {
      $ids = array_keys($entities[$entity_type]);
      $number_of_entities = count($ids);
    } else {
      $number_of_entities = 0;
    }

    if ($number_of_entities > 0) {
      $entities = entity_load($entity_type, $ids);
      $number_of_entities = count($entities);
    }

    if ($number_of_entities == 0) {
      $entity = (object)array(
        'is_new' => TRUE,
        'type' => $entity_type,
        'language' => $lang,
      );
      $entity->$bundle_key = $bundle;
    } else {
      $entity = array_shift($entities);
    }

    foreach ($data as $field => &$values) {
      $entity->{$field} = $values;
    }

    if (entity_save($entity_type, $entity) !== FALSE) {
      return $entity;
    }
    return FALSE;
  }

  /**
   * Autocomplete callback
   */
  static function autoComplete($config, $string) {
    return array($string);
  }
}
