<?php

class ReedsXMLReader extends ReedsReader {

  /**
   * Title of this reader
   */
  public function title() {
    return "XML Importer";
  }

  /**
   * Config form callback
   */
  static public function configForm(&$form, &$form_state, &$config) {
    $data = $config->data;
    $form['importer']['xpath_base'] = array(
      '#title' => t('XPath base expression'),
      '#description' => t('This can be for example /nodes/node when structure is &lt;nodes&gt;&lt;node&gt;'),
      '#type' => 'textfield',
      '#default_value' => $data['xpath_base'],
    );
    $form['importer']['example_xml'] = array(
      '#title' => t('Example XML'),
      '#description' => t('For autocompletion'),
      '#type' => 'textarea',
      '#default_value' => $data['example_xml'],
    );
    if (strlen($data['xpath_base']) > 0) {
      $form['importer']['#collapsed'] = TRUE;
    }
  }

  /**
   * Config form submit callback
   */
  static public function configFormSubmit(&$form, &$form_state, &$config) {
    $config->data['xpath_base'] = isset($form_state['input']['xpath_base']) ? $form_state['input']['xpath_base'] : '';
    $config->data['example_xml'] = isset($form_state['input']['example_xml']) ? $form_state['input']['example_xml'] : '';
  }

  /**
   * Config form item callback
   */
  static public function configFormItem($config, $key, $row) {
    return array(
      $key . '-xpath' => array(
        '#type' => 'textfield',
        '#autocomplete_path' => 'reeds/autocomplete/' . $config->machine_name,
        '#title' => 'XPath expression',
        '#default_value' => isset($row['xpath']) ? $row['xpath'] : '',
      )
    );
  }

  /**
   * config item form submit callback
   */
  static public function configFormItemSubmit($config, $input, $key, &$row) {
    $row['xpath'] = $input[$key . '-xpath'];
  }

  /**
   * Status form callback
   */
  static public function statusForm(&$form, &$form_state, &$config) {
    $form['file_path'] = array(
      '#title' => t('File'),
      '#description' => t('URL or file path.'),
      '#type' => 'textfield',
      '#default_value' => isset($config->data['file_path']) ? $config->data['file_path'] : '',
    );
  }

  /**
   * Status form submit
   */
  static public function statusFormSubmit(&$form, &$form_state, &$config) {
    $config->data['file_path'] = $form_state['input']['file_path'];
  }

  /**
   * Run reed configuration
   */
  static public function run(&$config) {

    $dom = new DomDocument();
    $dom->load($config->data['file_path']);
    $xpath = new DomXPath($dom);
    $items = $xpath->query($config->data['xpath_base']);

    // Gather entity info
    $lang = LANGUAGE_NONE;
    $entity_type = $config->entity_type;
    $bundle = $config->bundle;
    $info = entity_get_info($entity_type);
    $bundle_key = $info['entity keys']['bundle'];

    $new_items = 0;
    $updated_items = 0;
    $failed_items = 0;

    foreach ($items as $node) {

      // collect data of entity
      $data = array();
      $id_fields = array();

      foreach ($config->data['items'] as $item) {

        $field = $item['field'];
        $values = $xpath->query($item['xpath'], $node);

        foreach ($values as $value) {
          if (!isset($entity->{$field})) {
            $entity->{$field} = array(
              $lang => array()
            );
          }

          $field_info = field_info_instance($entity_type, $field, $bundle);
          if ($field_info) {
            $data[$field][$lang][] = array('value' => $value->nodeValue);
          } else {
            $data[$field] = $value->nodeValue;
          }

          if ($item['unique']) {
            $id_fields[$field] = $value->nodeValue;
          }
        }
      }

      $entity = self::createUpdateEntity($entity_type, $bundle, $lang, $id_fields, $data);

      if ($entity === FALSE) {
        $failed_items++;
      } else {
        if (isset($entity->is_new)) {
          $new_items++;
        }
        else {
          $updated_items++;
        }
      }
    }

    return array(
      'failed' => $failed_items,
      'updated' => $updated_items,
      'new' => $new_items,
    );
  }

  /**
   * Autocomplete callback
   */
  static function autoComplete($config, $string) {
    $doc = new DOMDocument;
    $doc->loadXML($config->data['example_xml']);
    $xpath = new DOMXPath($doc);
    $base = $config->data['xpath_base'];
    $path = $base . $string;
    $entries = $xpath->query($path . '/*');
    $result = array();
    foreach ($entries as $entry) {
      $val = $string . '/' . $entry->nodeName;
      $result[$val] = $val . ' (' . $base . $val . ')';
    }
    return $result;
  }

}
