<?php

function reeds_get_reader_by_type($type) {
  $reader_types = reeds_get_reader_types();
  return (isset($reader_types[$type])) ?
    $reader_types[$type]['class'] :
    FALSE;
}

/**
 * Menu title callback
 */
function reedscfg_title($op, $config) {
  switch ($op) {
    case 'edit': return 'Editing Reeds configuration ' . $config->name;
    case 'status': return 'Status of Reeds configuration ' . $config->name;
    case 'delete': return 'Confirm deletion of Reeds configuration ' . $config->name;
  }
  return '?';
}

/**
 * Configurations page callback
 */
function reeds_configurations() {
  $rows = array();

  $configs = reeds_get_configs();

  foreach ($configs as $machine_name => $config) {
    $rows[$machine_name] = array(
      'title' => l($config->name, 'admin/structure/reeds/' . $machine_name . '/status') . ' (' . $machine_name . ')',
      'type' => $config->type,
      'links' => l(t('Edit'), 'admin/structure/reeds/' . $machine_name . '/edit') . ' '
        . l(t('Delete'), 'admin/structure/reeds/' . $machine_name . '/delete'),
    );
  }

  $page['things'] = array(
    '#theme' => 'table',
    '#header' => array(
      'name' => t('Name'),
      'type' => t('Type'),
      'links' => t('Actions')
    ),
    '#rows' => $rows,
    '#suffix' => l(t('Create a new configuration'), 'admin/structure/reeds/add'),
    '#empty' => t('No items available'),
  );
  return $page;
}

/**
 * Configuration deletion callback
 */
function reeds_delete_form($form, &$form_state, $config = NULL) {

  $form['machine_name'] = array(
    '#title' => 'Machine name',
    '#type' => 'hidden',
    '#access' => FALSE,
    '#default_value' => $config->machine_name,
  );
  $form['cancel'] = array(
    '#type' => 'submit',
    '#value' => t('Cancel'),
  );
  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Delete'),
  );
  return $form;
}

/**
 * Implements hook_reeds_reader_info().
 */
function reeds_reeds_reader_info() {
  return array(
    'xml' => array('name' => 'XML Reader', 'class' => 'ReedsXMLReader'),
  );
}

/**
 * Get reader typesi
 */
function reeds_get_reader_types() {
  $types = module_invoke_all('reeds_reader_info');
  return $types;
}

/**
 * Delete configuration callback
 */
function reeds_delete_form_submit(&$form, &$form_state) {
  if ($form_state['input']['op'] == t('Delete')) {
    reedscfg_delete($form_state['values']['machine_name']);
  }
  drupal_goto("admin/structure/reeds");
}

/**
 * Configuration edit form
 */
function reeds_edit_form($form, &$form_state, $config = NULL) {
  $types = array('none' => t('- Select -'));
  $list = array_keys(entity_get_info());

  foreach ($list as $key) {
    $types[$key] = $key;
  }

  if (!isset($config)) {
    $config = (object)array(
      'name' => '',
      'machine_name' => '',
      'type' => 'none',
      'entity_type' => 'none',
      'bundle' => 'none',
      'data' => array(),
    );
  }

  if (!is_array($config->data)) {
    $config->data = array();
  }
  if (isset($form_state['config_edit'])) {
    $config = $form_state['config_edit'];
  } else {
    $form_state['config_edit'] = $config;
  }

  $form['#validate'] = array('reeds_edit_form_validate');


  $form['name'] = array(
    '#title' => 'Name',
    '#type' => 'textfield',
    '#maxlength' => 200,
    '#default_value' => $config->name,
  );

  $form['machine_name'] = array(
    '#title' => 'Machine name',
    '#type' => 'machine_name',
    '#maxlength' => 200,
    '#machine_name' => array(
      'exists' => 'reedscfg_exists',
    ),
    '#default_value' => $config->machine_name,
  );

  $form['importer'] = array(
    '#type' => 'fieldset',
    '#title' => t('Importer settings'),
    '#collapsible' => TRUE,
    '#collapsed' => FALSE,
  );

  $reader_types = reeds_get_reader_types();
  $list = array('none' => t('- Select -'));

  foreach ($reader_types as $key => $def) {
    $list[$key] = $def['name'];
  }

  $form['importer']['type'] = array(
    '#title' => t('Type'),
    '#type' => 'select',
    '#options' => $list,
    '#default_value' => $config->type,
    '#description' => t('The type of the importer defines how you want to read in the data.'),
  );

  $reader_type = FALSE;
  if (isset($reader_types[$config->type])) {
    $reader_type = $reader_types[$config->type]['class'];
  }

  $form['importer']['entity_type'] = array(
    '#title' => t('Entity type'),
    '#type' => 'select',
    '#options' => $types,
    '#default_value' => $config->entity_type,
    '#description' => t('Select what kind of entities you want to create / update.'),
  );

  $form['importer']['bundle'] = array(
    '#title' => 'Bundle',
    '#type' => 'select',
    '#options' => array('none' => t('- Select -')),
    '#disabled' => TRUE,
    '#description' => t('Select the bundle you wish to create.'),
  );

  if ($reader_type && isset($config->entity_type)  && $config->entity_type != 'none') {

    $list = array_keys(field_info_bundles($config->entity_type));
    $bundles = array();

    foreach ($list as $key) {
      $bundles[$key] = $key;
    }

    $form['importer']['bundle']['#disabled'] = FALSE;
    $form['importer']['bundle']['#options'] += $bundles;
    $form['importer']['bundle']['#default_value'] = $config->bundle;

    if (isset($config->bundle) && $config->bundle != 'none') {

      $reader_type::configForm($form, $form_state, $config);

      $form['fields'] = array(
        '#type' => 'fieldset',
        '#title' => t('Field mappings'),
        '#collapsible' => TRUE,
      );

      $fields = array('none' => t('- Delete -'));
      $list = entity_get_info($config->entity_type);
      foreach ($list['entity keys'] as $key) {
        $fields[$key] = $key;
      }

      $list = field_info_instances($config->entity_type, $config->bundle);
      foreach ($list as $key => $tmp) {
        $fields[$key] = $key;
      }

      $i = 0;
      foreach ($config->data['items'] as $i => $field) {
        $key = 'item-' . $i;
        $form['fields']['list-' . $key] =
          reeds_form_item($key, $field, $fields) +
          $reader_type::configFormItem($config, $key, $field);
      }

      $key = 'item-new';
      $form['fields']['item-max'] = array(
        '#type' => 'hidden',
        '#value' => $i + 1,
      );

      $fields['none'] = t('- Select -');

      $form['item-new'] = array(
        '#title' => 'Add new',
        '#type' => 'select',
        '#options' => $fields,
        '#value' => 'none',
        '#default_value' => 'none',
/*
          '#ajax' => array(
          'event' => 'change',
          'wrapper' => 'edit-fields',
          'callback' => 'reeds_form_ajax_callback',
          'method' => 'replace',
          'effect' => 'fade',
        ),
*/
      );

    }
  }

  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Submit'),
  );

  $form['save'] = array(
    '#type' => 'submit',
    '#value' => t('Save'),
  );

  return $form;
}

function reeds_form_ajax_callback($form, $form_state) {
  return $form['fields'];
}

/**
 * Field form part
 */
function reeds_form_item($key, $item, $fields) {
  return array(
    '#type' => 'fieldset',
    '#title' => 'Mapping to ' . $item['field'],
    '#collapsible' => TRUE,
    $key => array(
      '#title' => 'Target field',
      '#type' => 'select',
      '#options' => $fields,
      '#default_value' => $item['field'],
      '#value' => $item['field'],
    ),
    $key . '-unique' => array(
      '#title' => 'Unique',
      '#type' => 'checkbox',
      '#default_value' => isset($item['unique']) ? $item['unique'] : FALSE,
      '#value' => $item['unique'],
    ),
  );
}

/**
 * Validator for configuration edit form
 */
function reeds_edit_form_validate(&$form, &$form_state) {
}

/**
 * Configuration edit form submit callback
 */
function reeds_edit_form_submit(&$form, &$form_state) {

  $form_state['rebuild'] = TRUE;

  $data = $form_state['config_edit']->data;
  $data['items'] = array();

  $reader_types = reeds_get_reader_types();
  $reader_type = FALSE;
  if (isset($reader_types[$form_state['input']['type']])) {
    $reader_type = new $reader_types[$form_state['input']['type']]['class'];
  }

  if (isset($form_state['input']['item-max'])) {
    for ($i = 0; $i < $form_state['input']['item-max']; $i++) {
      $key = 'item-' . $i;
      if (isset($form_state['input'][$key]) && $form_state['input'][$key] != 'none') {
        $row = array(
          'field' => $form_state['input'][$key],
          'unique' => isset($form_state['input'][$key . '-unique']) ? TRUE : FALSE,
        );
        if ($reader_type) {
          $reader_type::configFormItemSubmit($form_state['config_edit'], $form_state['input'], $key, $row);
        }
        $data['items'][] = $row;
      }
    }
  }

  if (isset($form_state['input']['item-new']) && $form_state['input']['item-new'] != 'none') {
    $data['items'][] = array(
      'field' => $form_state['input']['item-new'],
    );
  }

  $form_state['config_edit'] = (object)array(
    'name'         => $form_state['input']['name'],
    'machine_name' => $form_state['input']['machine_name'],
    'type'         => $form_state['input']['type'],
    'entity_type'  => 'none',
    'bundle'       => 'none',
  );

  if (isset($form_state['input']['entity_type'])) {
    $form_state['config_edit']->entity_type = $form_state['input']['entity_type'];
    if (isset($form_state['input']['bundle'])) {
       $form_state['config_edit']->bundle = $form_state['input']['bundle'];
    }
  }

  $form_state['config_edit']->data = $data;

  if ($reader_type) {
    $reader_type::configFormSubmit($form, $form_state, $form_state['config_edit']);
  }

  if ($form_state['input']['op'] == t('Save')) {
    reedscfg_save($form_state['config_edit']);
    drupal_set_message(t('Saved'), 'status');
    drupal_goto('admin/structure/reeds');
  }
}

/**
 * Status form callback
 */
function reeds_status_form($form, &$form_state, $config) {

  $form['status'] = array(
    '#type' => 'fieldset',
    '#title' => $config->name,
    '#description' => t('Reader of type %type on entity_type %entity_type of bundle %bundle.',
      array(
        '%type' => $config->type,
        '%entity_type' => $config->entity_type,
        '%bundle' => $config->bundle,
      )
    )
  );

  $form_state['config'] = $config;

  $reader_type = reeds_get_reader_by_type($config->type);
  if ($reader_type) {
    $reader_type::statusForm($form,  $form_state, $config);
  }

  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Run now'),
  );

  return $form;
}

/**
 * Status form submit
 */
function reeds_status_form_submit(&$form, &$form_state) {
  $config = $form_state['config'];
  $reader_type = reeds_get_reader_by_type($config->type);
  if ($reader_type) {
    $reader_type::statusFormSubmit($form, $form_state, $config);
  }
  reedscfg_save($config);
  switch ($form_state['values']['op']) {
    case t('Run now'):
      if ($reader_type) {
        $status = $reader_type::run($config);
        drupal_set_message(t("Processed @new new entities and updated @updated entities.", array(
          "@new" => $status['new'],
          "@updated" => $status['updated'],
        )), "status");
        if ($status['failed'] > 0) {
          drupal_set_message(t("Failed on @failed entities.", array(
            "@failed" => $status['failed'],
          )), "failed");
        }
      }
      break;
  }
}


/**
 * Reeds autocomplete callback
 */
function reeds_autocomplete_callback($config) {

  $reader_types = reeds_get_reader_types();
  $reader_type = FALSE;

  $args = func_get_args();
  array_shift($args);
  $string = implode('/', $args);

  $matches = array();
  if (isset($reader_types[$config->type])) {
    $reader_type = new $reader_types[$config->type]['class'];
    $matches = $reader_type::autoComplete($config, $string);
  }

  return drupal_json_output($matches);
}
